# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-12-06 08:35
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='Theme',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('site', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='theme', to='sites.Site')),
            ],
        ),
    ]
